@@ -25,6 +25,17 @@ import re
 log = getLogger('butterfly')
 
 
+def parse_cert(cert):
+    user = None
+
+    for elt in cert['subject']:
+        user = dict(elt).get('commonName', None)
+        if user:
+            break
+
+    return user
+
+
 class User(object):
     def __init__(self, uid=None, name=None):
         if uid is None and not name:
